"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const ink_1 = require("ink");
const fs_1 = __importDefault(require("fs"));
const util_1 = require("util");
const split2_1 = __importDefault(require("split2"));
const readFile = util_1.promisify(fs_1.default.readFile);
const reducer = (prevState, action) => {
    switch (action.msg) {
        case 'GotResults':
            return Object.assign({}, prevState, { inProgress: prevState.inProgress.filter(href => href !== action.results.url), results: [...prevState.results, action.results] });
        case 'InProgress':
            return Object.assign({}, prevState, { inProgress: [...prevState.inProgress, action.href] });
        case 'DisplayInfo':
            return Object.assign({}, prevState, { info: [...prevState.info, action.description] });
        default:
            throw Error('Invalid action type');
    }
};
const StreamingReporter = ({}) => {
    // Consume stdin via context
    const { stdin } = react_1.useContext(ink_1.StdinContext);
    const [state, dispatch] = react_1.useReducer(reducer, {
        inProgress: [],
        results: [],
        info: [],
    });
    // Subscribe to stdin? (Wtf am I even doing here)
    react_1.useEffect(() => {
        // Use split so that each chunk is a separate line
        // This is because pino's output is NDJSON, enabling us to parse
        // and show each line at once!
        const splitStdin = stdin.pipe(split2_1.default());
        const onData = (data) => {
            try {
                const parsed = JSON.parse(data);
                const value = parsed;
                if (value.module === 'results') {
                    if (value.msg === 'GotResults') {
                        dispatch({ msg: 'GotResults', results: value.results });
                    }
                    if (value.msg === 'InProgress') {
                        dispatch({ msg: 'InProgress', href: value.href });
                    }
                    if (value.msg === 'DisplayInfo') {
                        dispatch({ msg: 'DisplayInfo', description: value.description });
                    }
                }
            }
            catch (err) { }
        };
        // Add the subscription
        splitStdin.on('data', onData);
        // Cleanup
        return () => {
            splitStdin.removeListener('data', onData);
        };
    }, [stdin]);
    return (react_1.default.createElement(Reporter, { inProgress: state.inProgress, results: state.results, info: state.info }));
};
const Reporter = props => {
    const { results, inProgress, info } = props;
    const pagesPassed = react_1.useMemo(() => results.filter(res => res.violations && res.violations.length === 0)
        .length, [results]);
    const hasPendingTests = inProgress.length;
    return (react_1.default.createElement(ink_1.Box, { flexDirection: "column" },
        react_1.default.createElement(ink_1.Static, null, results.map((result, i) => (react_1.default.createElement(ink_1.Box, { key: result.url, marginBottom: 2 },
            react_1.default.createElement(ResultRow, { result: result }))))),
        hasPendingTests ? (react_1.default.createElement(ink_1.Box, { flexDirection: "column" }, inProgress.map(href => (react_1.default.createElement(ink_1.Box, { key: href },
            react_1.default.createElement(ink_1.Color, { bgYellow: true, black: true }, "RUN"),
            ' ',
            href))))) : null,
        react_1.default.createElement(ink_1.Box, { marginBottom: 1 },
            react_1.default.createElement(ink_1.Text, null, "Pages:"),
            ' ',
            react_1.default.createElement(ink_1.Text, null,
                react_1.default.createElement(ink_1.Color, { green: true },
                    pagesPassed,
                    " had no violations"),
                ", ",
                pagesPassed,
                " of",
                ' ',
                results.length + inProgress.length,
                " total")),
        react_1.default.createElement(ink_1.Box, { flexDirection: "column" }, info.map(description => (react_1.default.createElement(ink_1.Box, { key: description, marginBottom: 1 },
            react_1.default.createElement(ink_1.Color, { blue: true }, description)))))));
};
const ResultRow = ({ result }) => {
    const hasViolations = result.violations && result.violations.length !== 0;
    return (react_1.default.createElement(ink_1.Box, { flexDirection: "column" },
        react_1.default.createElement(ink_1.Box, null,
            react_1.default.createElement(ink_1.Box, { marginRight: 1 }, hasViolations ? (react_1.default.createElement(ink_1.Color, { bgRed: true, black: true }, "FAIL")) : (react_1.default.createElement(ink_1.Color, { bgGreen: true, black: true }, "PASS"))),
            react_1.default.createElement(ink_1.Text, { bold: true }, result.url)),
        react_1.default.createElement(ink_1.Box, { flexDirection: "column" },
            hasViolations ? (react_1.default.createElement(ink_1.Box, null,
                react_1.default.createElement(ink_1.Color, { red: true },
                    react_1.default.createElement(ink_1.Text, null, "Violations: "),
                    react_1.default.createElement(ink_1.Text, null, result.violations.length)))) : null,
            react_1.default.createElement(ink_1.Box, { marginLeft: 1, flexDirection: "column" }, result.violations.map(violation => (react_1.default.createElement(ink_1.Box, { key: violation.id, marginBottom: 1 },
                "-",
                ' ',
                react_1.default.createElement(ink_1.Box, { flexDirection: "column" },
                    react_1.default.createElement(ink_1.Box, null,
                        react_1.default.createElement(ink_1.Text, null, violation.description)),
                    react_1.default.createElement(ink_1.Box, null,
                        react_1.default.createElement(ink_1.Text, null, violation.help)),
                    react_1.default.createElement(ink_1.Box, null,
                        "Learn more: ",
                        react_1.default.createElement(ink_1.Color, { underline: true }, violation.helpUrl)),
                    react_1.default.createElement(ink_1.Box, null,
                        "Nodes:",
                        ' ',
                        react_1.default.createElement(ink_1.Box, { flexDirection: "column" }, violation.nodes.map(node => (react_1.default.createElement(ink_1.Box, { key: node.target.join('') }, node.target.map(target => (react_1.default.createElement(ink_1.Color, { dim: true, key: target },
                            react_1.default.createElement(ink_1.Box, { key: target }, target))))))))))))))),
        react_1.default.createElement(ink_1.Box, null,
            react_1.default.createElement(ink_1.Color, { green: true },
                react_1.default.createElement(ink_1.Text, null, "Passes: "),
                react_1.default.createElement(ink_1.Text, null, result.passes.length))),
        react_1.default.createElement(ink_1.Box, null,
            react_1.default.createElement(ink_1.Color, { blue: true },
                react_1.default.createElement(ink_1.Text, null, "Incomplete: "),
                react_1.default.createElement(ink_1.Text, null, result.incomplete.length)))));
};
async function main(opts) {
    if (opts.reportSource.type === 'file') {
        // Read the data
        const reportData = await readFile(opts.reportSource.fileName, 'utf8').then(data => JSON.parse(data));
        ink_1.render(react_1.default.createElement(Reporter, { inProgress: [], results: reportData, info: [] }));
    }
    else if (opts.reportSource.type === 'stdin') {
        ink_1.render(react_1.default.createElement(StreamingReporter, null));
    }
}
// TODO: Figure out the exact process for streaming, if even available through CLI
function cliEntry(opts) {
    if (opts.streaming) {
        main({ reportSource: { type: 'stdin' } });
    }
    else if (opts.file) {
        main({ reportSource: { type: 'file', fileName: opts.file } });
    }
}
exports.cliEntry = cliEntry;
//# sourceMappingURL=report.js.map